import java.util.Hashtable;
import java.util.Map;

import javax.naming.AuthenticationException;
import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

public class LDAPQuest {
    public static void main(String[] args) {
        if (args.length < 7) {
            System.out.println("");
            System.out.println("USAGE: "+LDAPQuest.class.getName()+" <ldap-url> <ldap-admin-dn> <ldap-admin-password> <ldap-search-attr> <base-dn> <username> <password>");
            System.out.println("");
            System.out.println("IMPORTANT:");
            System.out.println("----------");
            System.out.println("For Microsoft Active Directory, ldap-search-attr should be sAMAccountName.");
            System.out.println("If an argument (e.g. password) has a space or a special character, enclose it in single quotes (').");
            System.out.println("");
            System.exit(1);
        }
        boolean debug = args.length == 8;
        System.out.println("LDAPQuest v2.0 " + (debug? "[DEBUG]" : ""));
        connect(trim(args[0]), trim(args[1]), trim(args[2]), trim(args[3]), trim(args[4]), trim(args[5]), trim(args[6]), debug);
        System.out.println("");
    }

    public static void connect(String url, String adminDN, String adminPassword, String searchAttr, String baseDn, String username, String password, boolean debug) 
    {
        LdapContext ctx = null;
        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.PROVIDER_URL, url);
        env.put("com.sun.jndi.ldap.read.timeout", "5000");
        env.put("com.sun.jndi.ldap.connect.timeout", "5000");
        env.put(Context.SECURITY_PRINCIPAL, adminDN);
        env.put(Context.SECURITY_CREDENTIALS, adminPassword);

        try
        {
            if (debug)
            {
               System.out.println("ENVIRONMENT ----------------------------------------------------------------------------");
               System.out.println("");
               for (Object key : env.keySet())
               {
                  System.out.println(key + "=" + env.get(key));
               }
               System.out.println("");
            }
            System.out.println("");
            System.out.println("1) Trying to connect to LDAP and bind as Admin...");
            System.out.println("----------------------------------------------------------------------------------------");
            ctx = new InitialLdapContext(env, null);
            System.out.println("SUCCESS: Admin authenticated successfully.");
            System.out.println("");
            System.out.println("2) Trying to authenticate end user...");
            System.out.println("----------------------------------------------------------------------------------------");
            search(ctx, searchAttr, baseDn, username, password, debug);
        }
        catch (AuthenticationException ae)
        {
            System.out.println("ERROR: Connected but authentication failed");
            if (debug)
            {
                ae.printStackTrace();
            }
        }
        catch (CommunicationException ex)
        {
            System.out.println("ERROR: Cannot connect to host or port. Check protocol/host/port.");
            if (debug)
            {
                ex.printStackTrace();
            }
        }
        catch (Exception ex)
        {
            System.out.println("ERROR: " + ex.getClass().getName() + ": " + ex.getMessage());
            if (debug)
            {
                ex.printStackTrace();
            }
        }
    }

    private static String trim(String s)
    {
        if (s != null) 
        {
            s = s.trim();
        }
        return s;
    }

    private static void search(LdapContext ctx, String searchAttr, String baseDn, String username, String password, boolean debug)
    {
        try 
        {
            System.out.println("SEARCHING for Username:" + username + " in Base DN:" + baseDn);

            String objectClass = "*";
            String searchFilter = "(&(objectClass=" + objectClass + ")(" + searchAttr + "=" + username + "))";
            SearchControls searchCtls = new SearchControls();
            searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            String searchBase = baseDn;
            NamingEnumeration answer = ctx.search(searchBase, searchFilter, searchCtls);
            boolean found = false;
            while (answer.hasMoreElements())
            {
                found = true;
                SearchResult sr = (SearchResult) answer.next();
                String dn = sr.getNameInNamespace();
                System.out.println("FOUND DN: " + dn);
                Attributes attrs = sr.getAttributes();
                if (attrs != null)
                {
                    if (debug) {
                        System.out.println("\tATTRIBUTES:" + attrs);
                    }
                    ctx.removeFromEnvironment(Context.SECURITY_PRINCIPAL);
                    ctx.removeFromEnvironment(Context.SECURITY_CREDENTIALS);
                    ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
                    ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, password);
                    try
                    {
                        System.out.println("");
                        System.out.println("3) Connecting as " + dn + "  with specified password...");
                        System.out.println("----------------------------------------------------------------------------------------");
                        ctx.reconnect(null);
                        System.out.println("SUCCESS: " + username + " authenticated successfully.");
                        return;
                    }
                    catch (AuthenticationException ex)
                    {
                        System.out.println("ERROR: incorrect password");
                        break;
                    }
                }
            }
            if (!found)
            {
                System.out.println("FAILURE: No matching user found");
            }
        }
        catch (Exception ex)
        {
            System.out.println("ERROR: " + ex.getClass().getName() + ": " + ex.getMessage());
        }
    }
}
